@@ -1,5 +1,6 @@
 from sqlobject import col, dberrors
 from sqlobject.compat import PY2
+from sqlobject.converters import registerConverter, StringLikeConverter
 from sqlobject.dbconnection import DBAPI
 
 
@@ -58,7 +59,7 @@ class MySQLConnection(DBAPI):
                         self.CR_SERVER_LOST = \
                             MySQLdb.constants.CR.CR_SERVER_LOST
                     self.ER_DUP_ENTRY = MySQLdb.constants.ER.DUP_ENTRY
-                elif driver == 'connector':
+                elif driver in ('connector', 'connector-python'):
                     import mysql.connector
                     self.module = mysql.connector
                     self.CR_SERVER_GONE_ERROR = \
@@ -66,6 +67,10 @@ class MySQLConnection(DBAPI):
                     self.CR_SERVER_LOST = \
                         mysql.connector.errorcode.CR_SERVER_LOST
                     self.ER_DUP_ENTRY = mysql.connector.errorcode.ER_DUP_ENTRY
+                    if driver == 'connector-python':
+                        self.connector_type = 'mysql.connector-python'
+                    else:
+                        self.connector_type = 'mysql.connector'
                 elif driver == 'oursql':
                     import oursql
                     self.module = oursql
@@ -91,7 +96,7 @@ class MySQLConnection(DBAPI):
                 else:
                     raise ValueError(
                         'Unknown MySQL driver "%s", '
-                        'expected mysqldb, connector, '
+                        'expected mysqldb, connector, connector-python, '
                         'oursql, pymysql, mariadb, '
                         'odbc, pyodbc or pypyodbc' % driver)
             except ImportError:
@@ -101,12 +106,14 @@ class MySQLConnection(DBAPI):
         else:
             raise ImportError(
                 'Cannot find a MySQL driver, tried %s' % drivers)
+
         self.host = host
         self.port = port or 3306
         self.db = db
         self.user = user
         self.password = password
         self.kw = {}
+
         for key in ("unix_socket", "init_command",
                     "read_default_file", "read_default_group", "conv"):
             if key in kw:
@@ -115,7 +122,7 @@ class MySQLConnection(DBAPI):
                     "client_flag", "local_infile"):
             if key in kw:
                 self.kw[key] = int(kw.pop(key))
-        if driver == 'connector':
+        if driver in ('connector', 'connector-python'):
             for key in ("ssl_key", "ssl_cert", "ssl_ca", "ssl_capath"):
                 if key in kw:
                     self.kw[key] = kw.pop(key)
@@ -151,6 +158,9 @@ class MySQLConnection(DBAPI):
         elif driver == 'mariadb':
             self.kw.pop("charset", None)
 
+        elif driver in ('connector', 'connector-python'):
+            registerConverter(bytes, ConnectorBytesConverter)
+
         global mysql_Bin
         if not PY2 and mysql_Bin is None:
             mysql_Bin = self.module.Binary
@@ -179,7 +189,7 @@ class MySQLConnection(DBAPI):
                     def character_set_name(self):
                         return dbEncoding + '_' + dbEncoding
                     Connection.character_set_name = character_set_name
-        if self.driver == 'connector':
+        if self.driver in ('connector', 'connector-python'):
             self.kw['consume_results'] = True
         try:
             if self.driver in ('odbc', 'pyodbc', 'pypyodbc'):
@@ -231,7 +241,7 @@ class MySQLConnection(DBAPI):
             try:
                 conn.autocommit(auto)
             except TypeError:
-                # mysql-connector has autocommit as a property
+                # mysql-connector{-python} has autocommit as a property
                 conn.autocommit = auto
 
     def _force_reconnect(self, conn):
@@ -246,7 +256,8 @@ class MySQLConnection(DBAPI):
             self.printDebug(conn, query, 'QueryR')
         dbEncoding = self.dbEncoding
         if dbEncoding and not isinstance(query, bytes) and (
-                self.driver in ('mysqldb', 'connector', 'oursql', 'mariadb')):
+                self.driver in ('mysqldb', 'connector', 'connector-python',
+                                'oursql', 'mariadb')):
             query = query.encode(dbEncoding, 'surrogateescape')
         # When a server connection is lost and a query is attempted, most of
         # the time the query will raise a SERVER_LOST exception, then at the
@@ -397,6 +408,9 @@ class MySQLConnection(DBAPI):
             # (SQLObject expected '')
             kw['notNone'] = (nullAllowed.upper() != 'YES' and True or False)
 
+            if not PY2 and isinstance(t, bytes):
+                t = t.decode('ascii')
+
             if default and t.startswith('int'):
                 kw['default'] = int(default)
             elif default and t.startswith('float'):
@@ -413,6 +427,8 @@ class MySQLConnection(DBAPI):
         return results
 
     def guessClass(self, t):
+        if not PY2 and isinstance(t, bytes):
+            t = t.decode('ascii')
         if t.startswith('int'):
             return col.IntCol, {}
         elif t.startswith('enum'):
@@ -469,10 +485,10 @@ class MySQLConnection(DBAPI):
             return col.Col, {}
 
     def listTables(self):
-        return [v[0] for v in self.queryAll("SHOW TABLES")]
+        return _decodeBytearrays(self.queryAll("SHOW TABLES"))
 
     def listDatabases(self):
-        return [v[0] for v in self.queryAll("SHOW DATABASES")]
+        return _decodeBytearrays(self.queryAll("SHOW DATABASES"))
 
     def _createOrDropDatabase(self, op="CREATE"):
         self.query('%s DATABASE %s' % (op, self.db))
@@ -528,3 +544,19 @@ class MySQLConnection(DBAPI):
             can_use_json_funcs = (server_version >= (5, 7, 0))
         self._can_use_json_funcs = can_use_json_funcs
         return can_use_json_funcs
+
+
+def ConnectorBytesConverter(value, db):
+    if not PY2:
+        # For PY2 this converter is called also for SQLite
+        assert db == 'mysql'
+        value = value.decode('latin1')
+    return StringLikeConverter(value, db)
+
+
+def _decodeBytearrays(v_list):
+    if not v_list:
+        return []
+    if not PY2 and isinstance(v_list[0][0], bytearray):
+        return [v[0].decode('ascii') for v in v_list]
+    return [v[0] for v in v_list]
