@@ -10,18 +10,40 @@ from sqlobject.dbconnection import DBAPI
 
 class ErrorMessage(str):
     def __new__(cls, e, append_msg=''):
-        obj = str.__new__(cls, e.args[0] + append_msg)
-        if hasattr(e, 'pgcode'):  # psycopg2 or psycopg2.errors
-            obj.code = getattr(e, 'pgcode', None)
-            obj.error = getattr(e, 'pgerror', None)
+        eargs0 = emessage = e.args[0]
+        if e.__module__.startswith('pg8000') \
+                and isinstance(e.args, tuple) and len(e.args) > 1:
+            # pg8000 =~ 1.12 for Python 3.4
+            ecode = e.args[2]
+            eerror = emessage = e.args[3]
+        elif e.__module__.startswith('pg8000') and isinstance(eargs0, dict):
+            # pg8000 =~ 1.13 for Python 2.7
+            # pg8000 for Python 3.5+
+            ecode = eargs0['C']
+            eerror = emessage = eargs0['M']
+        elif hasattr(e, 'pgcode'):  # psycopg2 or psycopg2.errors
+            ecode = getattr(e, 'pgcode', None)
+            eerror = getattr(e, 'pgerror', None)
         else:
-            obj.code = getattr(e, 'code', None)
-            obj.error = getattr(e, 'error', None)
+            ecode = getattr(e, 'code', None)
+            eerror = getattr(e, 'error', None)
+        obj = str.__new__(cls, emessage + append_msg)
+        obj.code = ecode
+        obj.error = eerror
         obj.module = e.__module__
         obj.exception = e.__class__.__name__
         return obj
 
 
+def _getuser():
+    # ``getuser()`` on w32 can raise ``ImportError``
+    # due to absent of ``pwd`` module.
+    try:
+        return getuser()
+    except ImportError:
+        return None
+
+
 class PostgresConnection(DBAPI):
 
     supportTransactions = True
@@ -161,7 +183,7 @@ class PostgresConnection(DBAPI):
                     dsn_dict["host"] = None
                     dsn_dict["unix_sock"] = host
                 if user is None:
-                    dsn_dict["user"] = getuser()
+                    dsn_dict["user"] = _getuser()
             self.dsn = dsn
         self.driver = driver
         self.unicodeCols = kw.pop('unicodeCols', False)
@@ -237,7 +259,8 @@ class PostgresConnection(DBAPI):
             raise dberrors.OperationalError(ErrorMessage(e))
         except self.module.IntegrityError as e:
             msg = ErrorMessage(e)
-            if getattr(e, 'code', -1) == '23505' or \
+            if getattr(msg, 'code', -1) == '23505' or \
+                    getattr(e, 'code', -1) == '23505' or \
                     getattr(e, 'pgcode', -1) == '23505' or \
                     getattr(e, 'sqlstate', -1) == '23505' or \
                     e.args[0] == '23505':
